/*
 * Copyright (c) 2022 Dmitry V. Levin <ldv@altlinux.org>
 * All rights reserved.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#include "config.h"

#ifdef HAVE___STAT64_TIME64

# define _GNU_SOURCE
# define _FILE_OFFSET_BITS 64
# define _TIME_BITS 64
# include "communicate.h"

# include <stdio.h>
# include <sys/stat.h>
# if HAVE_FTS_H
#  include <fts.h>
# endif

/* the following two header files are needed to include "wrapped.h" */
# ifdef HAVE_SYS_ACL_H
#  include <sys/acl.h>
# endif
# ifdef HAVE_SYS_CAPABILITY_H
#  include <sys/capability.h>
# endif

# include "wrapped.h"

static void send_get_stat64_time64(struct stat *st)
{
# ifndef FAKEROOT_FAKENET
  if (init_get_msg() == -1)
    return;
# endif
  struct fake_msg f = {
	  .id = stat_func,
	  .st = {
		  .mode  = st->st_mode,
		  .ino   = st->st_ino,
		  .uid   = st->st_uid,
		  .gid   = st->st_gid,
		  .dev   = st->st_dev,
		  .rdev  = st->st_rdev,
		  .nlink = st->st_nlink
	  }
  };
  send_get_fakem(&f);
  st->st_mode = f.st.mode;
  st->st_ino  = f.st.ino ;
  st->st_uid  = f.st.uid ;
  st->st_gid  = f.st.gid ;
  st->st_dev  = f.st.dev ;
  st->st_rdev = f.st.rdev;
}

int __lstat64_time64(const char *file_name, struct stat *st)
{
# ifdef LIBFAKEROOT_DEBUGGING
  if (fakeroot_debug) {
    fprintf(stderr, "__lstat64_time64 file_name %s\n", file_name);
# endif
  if (next___lstat64_time64(file_name, st))
    return -1;
  send_get_stat64_time64(st);
  return 0;
}

int __stat64_time64(const char *file_name, struct stat *st)
{
# ifdef LIBFAKEROOT_DEBUGGING
  if (fakeroot_debug)
    fprintf(stderr, "__stat64_time64 file_name %s\n", file_name);
# endif
  if (next___stat64_time64(file_name,st))
    return -1;
  send_get_stat64_time64(st);
  return 0;
}

int __fstat64_time64(int fd, struct stat *st)
{
# ifdef LIBFAKEROOT_DEBUGGING
  if (fakeroot_debug)
    fprintf(stderr, "__fstat64_time64 fd %d\n", fd);
# endif
  if (next___fstat64_time64(fd, st))
    return -1;
  send_get_stat64_time64(st);
  return 0;
}

int __fstatat64_time64(int dir_fd, const char *path,
		       struct stat *st, int flags)
{
  if (next___fstatat64_time64(dir_fd, path, st, flags))
    return -1;
  send_get_stat64_time64(st);
  return 0;
}

FTSENT *__fts64_read_time64(FTS *ftsp)
{
  FTSENT *r;
# ifdef LIBFAKEROOT_DEBUGGING
  if (fakeroot_debug)
    fprintf(stderr, "__fts64_read_time64\n");
# endif
  r = next___fts64_read_time64(ftsp);
  if (r && ((ftsp->fts_options & FTS_NOSTAT)
            || r->fts_info == FTS_NS || r->fts_info == FTS_NSOK))
    r->fts_statp = NULL;  /* Otherwise fts_statp may be a random pointer */
  if (r && r->fts_statp) {  /* Should we bother checking fts_info here? */
    send_get_stat64_time64(r->fts_statp);
  }
  return r;
}

FTSENT *__fts64_children_time64(FTS *ftsp, int options)
{
  FTSENT *first, *r;
# ifdef LIBFAKEROOT_DEBUGGING
  if (fakeroot_debug)
    fprintf(stderr, "__fts64_children_time64\n");
# endif
  first=next___fts64_children_time64(ftsp, options);
  for (r = first; r; r = r->fts_link) {
    if (r->fts_statp) {  /* Should we bother checking fts_info here? */
      send_get_stat64_time64(r->fts_statp);
    }
  }
  return first;
}

#endif /* HAVE___STAT64_TIME64 */
